/* --------------------------------------------------------------- */
/* Licensed Materials - Property of IBM                            */
/* 5724-S84                                                        */
/* (C) Copyright IBM Corp. 2008       All Rights Reserved          */
/* US Government Users Restricted Rights - Use, duplication or     */
/* disclosure restricted by GSA ADP Schedule Contract with         */
/* IBM Corp.                                                       */
/* --------------------------------------------------------------- */
/* PROLOG END TAG zYx                                              */
//$Revision: 1.13 $


#ifndef _MC_RAND_SFMT_H_
#define _MC_RAND_SFMT_H_


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifdef __SPU__

/** macro to compute SFMT working mem size in 128 bit vectors */
#define MC_RAND_SFMT_MEM_SIZE( mexp, max_count) \
   (40 + ((mexp/128)+1) +  \
   ((max_count >  ((mexp/128)+1)*2) ? max_count : ((mexp/128)+1)*2))


  typedef enum {
    MC_RAND_SFMT_OK            = 0,
    MC_RAND_SFMT_INVALID_MEXP  = 1,
    MC_RAND_SFMT_INVALID_LINE  = 2,
    MC_RAND_SFMT_DEFAULT_PS    = 3
  } mc_rand_sfmt_return_vals_t;

    

// Initialization routine
  int mc_rand_sfmt_init(unsigned int mexp, 
			 unsigned int line,
			 vector unsigned int *working_mem,
			 unsigned int seed);
  
  // Return single vectors of RNGs
  vector unsigned int mc_rand_sfmt_u4();
  
  vector float mc_rand_sfmt_minus1_to_1_f4();
  vector float mc_rand_sfmt_0_to_1_f4();

  vector double mc_rand_sfmt_0_to_1_d2e();
  vector double mc_rand_sfmt_minus1_to_1_d2e();

  vector double mc_rand_sfmt_0_to_1_d2();
  vector double mc_rand_sfmt_minus1_to_1_d2();
  
  // Return more than one vector of RNGs
  vector unsigned int* mc_rand_sfmt_array_u4( unsigned int count );
  
  vector float*  mc_rand_sfmt_0_to_1_array_f4( unsigned int count);
  vector float*  mc_rand_sfmt_minus1_to_1_array_f4( unsigned int count);

  vector double* mc_rand_sfmt_0_to_1_array_d2e( unsigned int count);
  vector double* mc_rand_sfmt_minus1_to_1_array_d2e( unsigned int count);

  vector double* mc_rand_sfmt_0_to_1_array_d2( unsigned int count);
  vector double* mc_rand_sfmt_minus1_to_1_array_d2( unsigned int count);

#endif

  // type for parameters
  typedef struct sfmt_struct {
    
    unsigned int mexp;	
    
    unsigned int pos1;	
    unsigned int sl1;	
    unsigned int sl2;	
    unsigned int sr1;	
    unsigned int sr2;	
    unsigned int msk1;	
    unsigned int msk2;	
    unsigned int msk3;	
    unsigned int msk4;	
    unsigned int parity1;	
    unsigned int parity2;	
    unsigned int parity3;	
    unsigned int parity4;	
    
    vector unsigned int alti_sl1; 
    vector unsigned int alti_sr1;
    vector unsigned int alti_msk;
    vector unsigned int alti_msk64;
    
    vector unsigned char alti_sl2_perm;
    vector unsigned char alti_sl2_perm64;
    vector unsigned char alti_sr2_perm;
    vector unsigned char alti_sr2_perm64;
    
  } sfmt_param_list_t __attribute__ (( aligned(128)));

#ifdef __PPU__

sfmt_param_list_t sfmt_params[192] = {


// 11213.csv
{ 11213, 68, 14, 3, 7, 3, 0xeffff7fbU, 0xffffffefU, 0xdfdfbfffU, 0x7fffdbfdU, 0x00000001U, 0x00000000U, 0xe8148000U, 0xd0c7afa3U, {14U, 14U, 14U, 14U}, {7U, 7U, 7U, 7U}, {0xeffff7fbU, 0xffffffefU, 0xdfdfbfffU, 0x7fffdbfdU}, {0xffffffefU, 0xeffff7fbU, 0x7fffdbfdU, 0xdfdfbfffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 73, 11, 3, 2, 5, 0xff6ffcffU, 0xbff77dffU, 0x77dfffffU, 0xfbfffffbU, 0x00000001U, 0x00000000U, 0xbcaaa000U, 0x278c70beU, {11U, 11U, 11U, 11U}, {2U, 2U, 2U, 2U}, {0xff6ffcffU, 0xbff77dffU, 0x77dfffffU, 0xfbfffffbU}, {0xbff77dffU, 0xff6ffcffU, 0xfbfffffbU, 0x77dfffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 11213, 72, 13, 3, 1, 1, 0x7ff7ffffU, 0xafffcddeU, 0xeffffeffU, 0xffffffffU, 0x00000001U, 0x00000000U, 0x03dc0000U, 0x1f9394f8U, {13U, 13U, 13U, 13U}, {1U, 1U, 1U, 1U}, {0x7ff7ffffU, 0xafffcddeU, 0xeffffeffU, 0xffffffffU}, {0xafffcddeU, 0x7ff7ffffU, 0xffffffffU, 0xeffffeffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 63, 22, 1, 13, 1, 0xf5fbfffbU, 0xffffffbfU, 0xdf7ffcb7U, 0xdffff7ffU, 0x00000001U, 0x00000000U, 0xf73ca000U, 0x2ad648b9U, {22U, 22U, 22U, 22U}, {13U, 13U, 13U, 13U}, {0xf5fbfffbU, 0xffffffbfU, 0xdf7ffcb7U, 0xdffff7ffU}, {0xffffffbfU, 0xf5fbfffbU, 0xdffff7ffU, 0xdf7ffcb7U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 73, 12, 3, 9, 1, 0xddffdff7U, 0x6baffff7U, 0x7bd5ffffU, 0xefd3fbffU, 0x00000001U, 0x00000000U, 0xf75e8000U, 0xb53672e0U, {12U, 12U, 12U, 12U}, {9U, 9U, 9U, 9U}, {0xddffdff7U, 0x6baffff7U, 0x7bd5ffffU, 0xefd3fbffU}, {0x6baffff7U, 0xddffdff7U, 0xefd3fbffU, 0x7bd5ffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 48, 10, 3, 9, 5, 0x5dffefcdU, 0xc6ffffdfU, 0xfe77bfffU, 0xffefefffU, 0x00000001U, 0x00000000U, 0x6f4ea000U, 0x1b8db4baU, {10U, 10U, 10U, 10U}, {9U, 9U, 9U, 9U}, {0x5dffefcdU, 0xc6ffffdfU, 0xfe77bfffU, 0xffefefffU}, {0xc6ffffdfU, 0x5dffefcdU, 0xffefefffU, 0xfe77bfffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 11213, 45, 13, 3, 10, 5, 0xfbffd7ffU, 0xddf7f7e7U, 0x9bffe71fU, 0xff7ec3ffU, 0x00000001U, 0x00000000U, 0x830e0000U, 0xcd4127afU, {13U, 13U, 13U, 13U}, {10U, 10U, 10U, 10U}, {0xfbffd7ffU, 0xddf7f7e7U, 0x9bffe71fU, 0xff7ec3ffU}, {0xddf7f7e7U, 0xfbffd7ffU, 0xff7ec3ffU, 0x9bffe71fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 11213, 58, 7, 7, 5, 1, 0x7defffffU, 0x7d9bffffU, 0xfdbff7eeU, 0xdfffffffU, 0x00000001U, 0x00000000U, 0x5b202000U, 0x95b190fdU, {7U, 7U, 7U, 7U}, {5U, 5U, 5U, 5U}, {0x7defffffU, 0x7d9bffffU, 0xfdbff7eeU, 0xdfffffffU}, {0x7d9bffffU, 0x7defffffU, 0xdfffffffU, 0xfdbff7eeU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 86, 19, 1, 5, 1, 0xfffffebfU, 0xfdeffadfU, 0x7fb71fcfU, 0xffff5fffU, 0x00000001U, 0x00000000U, 0x953e8000U, 0xf7bd34c4U, {19U, 19U, 19U, 19U}, {5U, 5U, 5U, 5U}, {0xfffffebfU, 0xfdeffadfU, 0x7fb71fcfU, 0xffff5fffU}, {0xfdeffadfU, 0xfffffebfU, 0xffff5fffU, 0x7fb71fcfU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 84, 22, 1, 7, 1, 0xfff7effeU, 0xbfb7afffU, 0xb7dfffffU, 0xbf7f7fefU, 0x00000001U, 0x00000000U, 0xe1d70000U, 0x02717d9dU, {22U, 22U, 22U, 22U}, {7U, 7U, 7U, 7U}, {0xfff7effeU, 0xbfb7afffU, 0xb7dfffffU, 0xbf7f7fefU}, {0xbfb7afffU, 0xfff7effeU, 0xbf7f7fefU, 0xb7dfffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 33, 6, 7, 7, 1, 0xfffff9ffU, 0xfb7fffffU, 0xebeffdfeU, 0xfbfbffffU, 0x00000001U, 0x00000000U, 0x8a3d0800U, 0x026e932dU, {6U, 6U, 6U, 6U}, {7U, 7U, 7U, 7U}, {0xfffff9ffU, 0xfb7fffffU, 0xebeffdfeU, 0xfbfbffffU}, {0xfb7fffffU, 0xfffff9ffU, 0xfbfbffffU, 0xebeffdfeU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 22, 14, 7, 7, 1, 0xfffefbf7U, 0xeffffb7eU, 0xfefdfbbfU, 0xe7ffffdfU, 0x00000001U, 0x00000000U, 0x5c142000U, 0x0aaefe70U, {14U, 14U, 14U, 14U}, {7U, 7U, 7U, 7U}, {0xfffefbf7U, 0xeffffb7eU, 0xfefdfbbfU, 0xe7ffffdfU}, {0xeffffb7eU, 0xfffefbf7U, 0xe7ffffdfU, 0xfefdfbbfU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 79, 22, 1, 17, 1, 0xffbf7ff7U, 0x9dffdfdfU, 0xdfef97bfU, 0xdfeebf7fU, 0x00000001U, 0x00000000U, 0x68cbe000U, 0x1c20bdd1U, {22U, 22U, 22U, 22U}, {17U, 17U, 17U, 17U}, {0xffbf7ff7U, 0x9dffdfdfU, 0xdfef97bfU, 0xdfeebf7fU}, {0x9dffdfdfU, 0xffbf7ff7U, 0xdfeebf7fU, 0xdfef97bfU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 77, 9, 3, 10, 3, 0xfffffffeU, 0x3feefbffU, 0xf7fefeffU, 0xf5bc9fffU, 0x00000001U, 0x00000000U, 0x73b20000U, 0xb134ef9cU, {9U, 9U, 9U, 9U}, {10U, 10U, 10U, 10U}, {0xfffffffeU, 0x3feefbffU, 0xf7fefeffU, 0xf5bc9fffU}, {0x3feefbffU, 0xfffffffeU, 0xf5bc9fffU, 0xf7fefeffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 45, 17, 1, 6, 3, 0xf77bbfefU, 0xbcf7bfdfU, 0xbffff7e7U, 0xfffaff7bU, 0x00000001U, 0x00000000U, 0xce408000U, 0x1344dad1U, {17U, 17U, 17U, 17U}, {6U, 6U, 6U, 6U}, {0xf77bbfefU, 0xbcf7bfdfU, 0xbffff7e7U, 0xfffaff7bU}, {0xbcf7bfdfU, 0xf77bbfefU, 0xfffaff7bU, 0xbffff7e7U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 41, 13, 1, 4, 3, 0xfb5d7fffU, 0xfbff7bffU, 0xf7ff677fU, 0xc67fffbfU, 0x00000001U, 0x00000000U, 0x0eb18000U, 0xafe068e2U, {13U, 13U, 13U, 13U}, {4U, 4U, 4U, 4U}, {0xfb5d7fffU, 0xfbff7bffU, 0xf7ff677fU, 0xc67fffbfU}, {0xfbff7bffU, 0xfb5d7fffU, 0xc67fffbfU, 0xf7ff677fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 69, 6, 3, 3, 1, 0xeffdfdffU, 0xf9bffbffU, 0xdff1ffffU, 0xf7f8dfffU, 0x00000001U, 0x00000000U, 0x83409000U, 0xc4651bf2U, {6U, 6U, 6U, 6U}, {3U, 3U, 3U, 3U}, {0xeffdfdffU, 0xf9bffbffU, 0xdff1ffffU, 0xf7f8dfffU}, {0xf9bffbffU, 0xeffdfdffU, 0xf7f8dfffU, 0xdff1ffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 48, 12, 3, 17, 1, 0xde5fffbfU, 0xf7d5f7beU, 0xfff7757dU, 0xfffeef79U, 0x00000001U, 0x00000000U, 0xa25c0200U, 0xa9cb6a61U, {12U, 12U, 12U, 12U}, {17U, 17U, 17U, 17U}, {0xde5fffbfU, 0xf7d5f7beU, 0xfff7757dU, 0xfffeef79U}, {0xf7d5f7beU, 0xde5fffbfU, 0xfffeef79U, 0xfff7757dU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 9, 18, 5, 15, 1, 0x5fe7eff7U, 0xdbffdff3U, 0xdbff6ff7U, 0xbb377fffU, 0x00000001U, 0x00000000U, 0xa0b64800U, 0xc3969fa3U, {18U, 18U, 18U, 18U}, {15U, 15U, 15U, 15U}, {0x5fe7eff7U, 0xdbffdff3U, 0xdbff6ff7U, 0xbb377fffU}, {0xdbffdff3U, 0x5fe7eff7U, 0xbb377fffU, 0xdbff6ff7U}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 48, 11, 7, 17, 1, 0xffd7ebffU, 0xefffff7bU, 0xefbfffbdU, 0xf7f7efaeU, 0x00000001U, 0x00000000U, 0x76adc000U, 0xdd5c6c66U, {11U, 11U, 11U, 11U}, {17U, 17U, 17U, 17U}, {0xffd7ebffU, 0xefffff7bU, 0xefbfffbdU, 0xf7f7efaeU}, {0xefffff7bU, 0xffd7ebffU, 0xf7f7efaeU, 0xefbfffbdU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 36, 5, 7, 17, 1, 0xccfe7ffeU, 0xfadfffffU, 0xfffffdffU, 0xffefefffU, 0x00000001U, 0x00000000U, 0x637cd000U, 0xedd65806U, {5U, 5U, 5U, 5U}, {17U, 17U, 17U, 17U}, {0xccfe7ffeU, 0xfadfffffU, 0xfffffdffU, 0xffefefffU}, {0xfadfffffU, 0xccfe7ffeU, 0xffefefffU, 0xfffffdffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 12, 7, 3, 11, 1, 0xfffbe5fbU, 0xfdfdf7dfU, 0xfffdf7ffU, 0xfffffffbU, 0x00000001U, 0x00000000U, 0x43f60800U, 0x4f337b1dU, {7U, 7U, 7U, 7U}, {11U, 11U, 11U, 11U}, {0xfffbe5fbU, 0xfdfdf7dfU, 0xfffdf7ffU, 0xfffffffbU}, {0xfdfdf7dfU, 0xfffbe5fbU, 0xfffffffbU, 0xfffdf7ffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 37, 21, 3, 4, 5, 0x76fe7fffU, 0x79ffffffU, 0x7ff7dfffU, 0xbbfbefffU, 0x00000001U, 0x00000000U, 0x2aa69000U, 0xa1bd24ddU, {21U, 21U, 21U, 21U}, {4U, 4U, 4U, 4U}, {0x76fe7fffU, 0x79ffffffU, 0x7ff7dfffU, 0xbbfbefffU}, {0x79ffffffU, 0x76fe7fffU, 0xbbfbefffU, 0x7ff7dfffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 11213, 85, 19, 7, 10, 1, 0xfffdfbffU, 0xffbffa3fU, 0xffbffa2fU, 0xffdfffbfU, 0x00000001U, 0x00000000U, 0xfe0c8800U, 0xceed51f4U, {19U, 19U, 19U, 19U}, {10U, 10U, 10U, 10U}, {0xfffdfbffU, 0xffbffa3fU, 0xffbffa2fU, 0xffdfffbfU}, {0xffbffa3fU, 0xfffdfbffU, 0xffdfffbfU, 0xffbffa2fU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 42, 13, 7, 21, 1, 0xff3ebfbeU, 0xff7fffdeU, 0x5ffffbefU, 0xfffefbfbU, 0x00000001U, 0x00000000U, 0x0443a000U, 0x35321b2aU, {13U, 13U, 13U, 13U}, {21U, 21U, 21U, 21U}, {0xff3ebfbeU, 0xff7fffdeU, 0x5ffffbefU, 0xfffefbfbU}, {0xff7fffdeU, 0xff3ebfbeU, 0xfffefbfbU, 0x5ffffbefU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 38, 16, 3, 1, 5, 0xfdf7dbffU, 0xfbffdffbU, 0xfbffbfd9U, 0xfffffdbdU, 0x00000001U, 0x00000000U, 0x16d92000U, 0xacd93bf9U, {16U, 16U, 16U, 16U}, {1U, 1U, 1U, 1U}, {0xfdf7dbffU, 0xfbffdffbU, 0xfbffbfd9U, 0xfffffdbdU}, {0xfbffdffbU, 0xfdf7dbffU, 0xfffffdbdU, 0xfbffbfd9U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 11213, 80, 3, 3, 1, 3, 0xf77efedfU, 0xf7fffbebU, 0xcbffffe7U, 0xfdfffbffU, 0x00000001U, 0x00000000U, 0xbca5a000U, 0x45e06438U, {3U, 3U, 3U, 3U}, {1U, 1U, 1U, 1U}, {0xf77efedfU, 0xf7fffbebU, 0xcbffffe7U, 0xfdfffbffU}, {0xf7fffbebU, 0xf77efedfU, 0xfdfffbffU, 0xcbffffe7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 37, 21, 7, 6, 1, 0xffcffdffU, 0xfedffdffU, 0xfcbffffbU, 0xffeffebfU, 0x00000001U, 0x00000000U, 0xf8270000U, 0xf6fdf776U, {21U, 21U, 21U, 21U}, {6U, 6U, 6U, 6U}, {0xffcffdffU, 0xfedffdffU, 0xfcbffffbU, 0xffeffebfU}, {0xfedffdffU, 0xffcffdffU, 0xffeffebfU, 0xfcbffffbU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 14, 11, 1, 21, 1, 0xffffffbdU, 0xff7feffeU, 0xb6ffd7ffU, 0x97f9fe7fU, 0x00000001U, 0x00000000U, 0x22d14000U, 0xf393e8a4U, {11U, 11U, 11U, 11U}, {21U, 21U, 21U, 21U}, {0xffffffbdU, 0xff7feffeU, 0xb6ffd7ffU, 0x97f9fe7fU}, {0xff7feffeU, 0xffffffbdU, 0x97f9fe7fU, 0xb6ffd7ffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 16, 27, 1, 5, 3, 0xdfffef7fU, 0xeff97fdfU, 0xfff3fcfaU, 0xfe7feeffU, 0x00000001U, 0x00000000U, 0x16f44800U, 0x24ce4227U, {27U, 27U, 27U, 27U}, {5U, 5U, 5U, 5U}, {0xdfffef7fU, 0xeff97fdfU, 0xfff3fcfaU, 0xfe7feeffU}, {0xeff97fdfU, 0xdfffef7fU, 0xfe7feeffU, 0xfff3fcfaU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 11213, 85, 5, 7, 14, 1, 0xffbfeffeU, 0xfbfddbebU, 0xfbdfffffU, 0xbfffffffU, 0x00000001U, 0x00000000U, 0xa7420000U, 0xea7b6ca0U, {5U, 5U, 5U, 5U}, {14U, 14U, 14U, 14U}, {0xffbfeffeU, 0xfbfddbebU, 0xfbdfffffU, 0xbfffffffU}, {0xfbfddbebU, 0xffbfeffeU, 0xbfffffffU, 0xfbdfffffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 11213, 13, 15, 5, 10, 3, 0xfffaffffU, 0xeffef9ffU, 0xfe7e9bdbU, 0xff7ffbfeU, 0x00000001U, 0x00000000U, 0x42c21400U, 0xfde64440U, {15U, 15U, 15U, 15U}, {10U, 10U, 10U, 10U}, {0xfffaffffU, 0xeffef9ffU, 0xfe7e9bdbU, 0xff7ffbfeU}, {0xeffef9ffU, 0xfffaffffU, 0xff7ffbfeU, 0xfe7e9bdbU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 

// 1279.csv
{ 1279, 3, 18, 1, 1, 1, 0xfffffffbU, 0x37f5effbU, 0xb3fdaff9U, 0xfffffd7dU, 0x00000001U, 0x00000000U, 0x00000000U, 0x40000000U, {18U, 18U, 18U, 18U}, {1U, 1U, 1U, 1U}, {0xfffffffbU, 0x37f5effbU, 0xb3fdaff9U, 0xfffffd7dU}, {0x37f5effbU, 0xfffffffbU, 0xfffffd7dU, 0xb3fdaff9U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 7, 14, 3, 5, 1, 0xf7fefffdU, 0x7fefcfffU, 0xaff3ef3fU, 0xb5ffff7fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x20000000U, {14U, 14U, 14U, 14U}, {5U, 5U, 5U, 5U}, {0xf7fefffdU, 0x7fefcfffU, 0xaff3ef3fU, 0xb5ffff7fU}, {0x7fefcfffU, 0xf7fefffdU, 0xb5ffff7fU, 0xaff3ef3fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 6, 21, 1, 17, 1, 0xfffbbfffU, 0xfffbdffbU, 0xddfffdffU, 0x57ffffbfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {21U, 21U, 21U, 21U}, {17U, 17U, 17U, 17U}, {0xfffbbfffU, 0xfffbdffbU, 0xddfffdffU, 0x57ffffbfU}, {0xfffbdffbU, 0xfffbbfffU, 0x57ffffbfU, 0xddfffdffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 8, 19, 1, 15, 1, 0xfbefeffdU, 0xffefffffU, 0xdfef7effU, 0xefeeffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {19U, 19U, 19U, 19U}, {15U, 15U, 15U, 15U}, {0xfbefeffdU, 0xffefffffU, 0xdfef7effU, 0xefeeffffU}, {0xffefffffU, 0xfbefeffdU, 0xefeeffffU, 0xdfef7effU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 3, 20, 1, 5, 1, 0xfddfbf9fU, 0xeefffb37U, 0xfff7df7bU, 0xfffefbefU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {20U, 20U, 20U, 20U}, {5U, 5U, 5U, 5U}, {0xfddfbf9fU, 0xeefffb37U, 0xfff7df7bU, 0xfffefbefU}, {0xeefffb37U, 0xfddfbf9fU, 0xfffefbefU, 0xfff7df7bU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 5, 15, 3, 12, 1, 0xfddf9f97U, 0xf97fbff7U, 0x7fbfbff7U, 0xbfbf1fffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x20000000U, {15U, 15U, 15U, 15U}, {12U, 12U, 12U, 12U}, {0xfddf9f97U, 0xf97fbff7U, 0x7fbfbff7U, 0xbfbf1fffU}, {0xf97fbff7U, 0xfddf9f97U, 0xbfbf1fffU, 0x7fbfbff7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 6, 13, 3, 21, 1, 0xfffbffffU, 0xfffe96efU, 0xedd7ff2fU, 0xf5f7f8edU, 0x00000001U, 0x00000000U, 0x00000000U, 0x40000000U, {13U, 13U, 13U, 13U}, {21U, 21U, 21U, 21U}, {0xfffbffffU, 0xfffe96efU, 0xedd7ff2fU, 0xf5f7f8edU}, {0xfffe96efU, 0xfffbffffU, 0xf5f7f8edU, 0xedd7ff2fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 6, 17, 3, 5, 3, 0xafffffffU, 0xff37bffbU, 0xe7bdffddU, 0xfffdffddU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {17U, 17U, 17U, 17U}, {5U, 5U, 5U, 5U}, {0xafffffffU, 0xff37bffbU, 0xe7bdffddU, 0xfffdffddU}, {0xff37bffbU, 0xafffffffU, 0xfffdffddU, 0xe7bdffddU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 2, 18, 1, 11, 1, 0xdeffffefU, 0xdeb7effbU, 0xdf77defeU, 0xdfdfd5cfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x40000000U, {18U, 18U, 18U, 18U}, {11U, 11U, 11U, 11U}, {0xdeffffefU, 0xdeb7effbU, 0xdf77defeU, 0xdfdfd5cfU}, {0xdeb7effbU, 0xdeffffefU, 0xdfdfd5cfU, 0xdf77defeU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 8, 21, 1, 3, 7, 0xfffdfdfeU, 0xfbbffeffU, 0xef6fffffU, 0xff7f76efU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {21U, 21U, 21U, 21U}, {3U, 3U, 3U, 3U}, {0xfffdfdfeU, 0xfbbffeffU, 0xef6fffffU, 0xff7f76efU}, {0xfbbffeffU, 0xfffdfdfeU, 0xff7f76efU, 0xef6fffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 1279, 1, 25, 1, 6, 1, 0xeff6fdedU, 0xfffffdfcU, 0xffffbfffU, 0xcfefffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {25U, 25U, 25U, 25U}, {6U, 6U, 6U, 6U}, {0xeff6fdedU, 0xfffffdfcU, 0xffffbfffU, 0xcfefffffU}, {0xfffffdfcU, 0xeff6fdedU, 0xcfefffffU, 0xffffbfffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 1, 6, 3, 13, 3, 0xffeecff3U, 0xffffedffU, 0xdbf7bfffU, 0xd1ffdffdU, 0x00000001U, 0x00000000U, 0x00000000U, 0x20000000U, {6U, 6U, 6U, 6U}, {13U, 13U, 13U, 13U}, {0xffeecff3U, 0xffffedffU, 0xdbf7bfffU, 0xd1ffdffdU}, {0xffffedffU, 0xffeecff3U, 0xd1ffdffdU, 0xdbf7bfffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 8, 6, 3, 9, 1, 0xeffff777U, 0xeefbfdffU, 0xfeffffffU, 0xfff7fffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {6U, 6U, 6U, 6U}, {9U, 9U, 9U, 9U}, {0xeffff777U, 0xeefbfdffU, 0xfeffffffU, 0xfff7fffeU}, {0xeefbfdffU, 0xeffff777U, 0xfff7fffeU, 0xfeffffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 3, 13, 3, 12, 5, 0xffdf7ff7U, 0xfff4ffefU, 0xedfe5fefU, 0xff7fdfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {13U, 13U, 13U, 13U}, {12U, 12U, 12U, 12U}, {0xffdf7ff7U, 0xfff4ffefU, 0xedfe5fefU, 0xff7fdfffU}, {0xfff4ffefU, 0xffdf7ff7U, 0xff7fdfffU, 0xedfe5fefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 1279, 1, 19, 3, 4, 1, 0xff7fffffU, 0xfffbfd7fU, 0xffefffeaU, 0xfddfdfdeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x40000000U, {19U, 19U, 19U, 19U}, {4U, 4U, 4U, 4U}, {0xff7fffffU, 0xfffbfd7fU, 0xffefffeaU, 0xfddfdfdeU}, {0xfffbfd7fU, 0xff7fffffU, 0xfddfdfdeU, 0xffefffeaU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 8, 6, 3, 3, 5, 0xf7f9f7ddU, 0xb7f7beffU, 0xf7ffdf7fU, 0xffdff7f9U, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {6U, 6U, 6U, 6U}, {3U, 3U, 3U, 3U}, {0xf7f9f7ddU, 0xb7f7beffU, 0xf7ffdf7fU, 0xffdff7f9U}, {0xb7f7beffU, 0xf7f9f7ddU, 0xffdff7f9U, 0xf7ffdf7fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 1279, 6, 5, 3, 3, 3, 0xdfefeffdU, 0xdfffffbfU, 0xdeffb9f7U, 0xfffffef7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {5U, 5U, 5U, 5U}, {3U, 3U, 3U, 3U}, {0xdfefeffdU, 0xdfffffbfU, 0xdeffb9f7U, 0xfffffef7U}, {0xdfffffbfU, 0xdfefeffdU, 0xfffffef7U, 0xdeffb9f7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 2, 19, 3, 13, 3, 0xfff97fffU, 0x6bfff7feU, 0xdafffb7dU, 0xefffffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {19U, 19U, 19U, 19U}, {13U, 13U, 13U, 13U}, {0xfff97fffU, 0x6bfff7feU, 0xdafffb7dU, 0xefffffffU}, {0x6bfff7feU, 0xfff97fffU, 0xefffffffU, 0xdafffb7dU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 7, 13, 7, 2, 1, 0x7df7ffffU, 0xdfd7ff7fU, 0xf7f2b3ddU, 0xe77ffddfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {13U, 13U, 13U, 13U}, {2U, 2U, 2U, 2U}, {0x7df7ffffU, 0xdfd7ff7fU, 0xf7f2b3ddU, 0xe77ffddfU}, {0xdfd7ff7fU, 0x7df7ffffU, 0xe77ffddfU, 0xf7f2b3ddU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 8, 6, 7, 5, 1, 0xdf7ffaffU, 0xdffaffffU, 0xfdd5bfefU, 0xffd7bfddU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {6U, 6U, 6U, 6U}, {5U, 5U, 5U, 5U}, {0xdf7ffaffU, 0xdffaffffU, 0xfdd5bfefU, 0xffd7bfddU}, {0xdffaffffU, 0xdf7ffaffU, 0xffd7bfddU, 0xfdd5bfefU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 6, 13, 1, 1, 7, 0xfddbffffU, 0xebffdffcU, 0xffeff7fdU, 0xdfdffeffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x10000000U, {13U, 13U, 13U, 13U}, {1U, 1U, 1U, 1U}, {0xfddbffffU, 0xebffdffcU, 0xffeff7fdU, 0xdfdffeffU}, {0xebffdffcU, 0xfddbffffU, 0xdfdffeffU, 0xffeff7fdU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 1279, 7, 27, 1, 2, 1, 0xfffb67ffU, 0xdefbe67bU, 0xef1fffffU, 0x3effff8fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {27U, 27U, 27U, 27U}, {2U, 2U, 2U, 2U}, {0xfffb67ffU, 0xdefbe67bU, 0xef1fffffU, 0x3effff8fU}, {0xdefbe67bU, 0xfffb67ffU, 0x3effff8fU, 0xef1fffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 4, 26, 1, 3, 1, 0xfffdfffaU, 0xffffbfebU, 0xfffdff9aU, 0xbfffffdfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {26U, 26U, 26U, 26U}, {3U, 3U, 3U, 3U}, {0xfffdfffaU, 0xffffbfebU, 0xfffdff9aU, 0xbfffffdfU}, {0xffffbfebU, 0xfffdfffaU, 0xbfffffdfU, 0xfffdff9aU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 7, 11, 3, 12, 5, 0xfdfeedffU, 0xdffcfff7U, 0x7bdffffeU, 0xffdfffeaU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {11U, 11U, 11U, 11U}, {12U, 12U, 12U, 12U}, {0xfdfeedffU, 0xdffcfff7U, 0x7bdffffeU, 0xffdfffeaU}, {0xdffcfff7U, 0xfdfeedffU, 0xffdfffeaU, 0x7bdffffeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 1279, 6, 5, 7, 11, 1, 0x4fdffdffU, 0xc7feff7fU, 0xf5ffbe7fU, 0x3fddff97U, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {5U, 5U, 5U, 5U}, {11U, 11U, 11U, 11U}, {0x4fdffdffU, 0xc7feff7fU, 0xf5ffbe7fU, 0x3fddff97U}, {0xc7feff7fU, 0x4fdffdffU, 0x3fddff97U, 0xf5ffbe7fU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 1279, 8, 13, 1, 5, 3, 0xffdffbaeU, 0x7ffbff3fU, 0xffffab5bU, 0xe7fff35bU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {13U, 13U, 13U, 13U}, {5U, 5U, 5U, 5U}, {0xffdffbaeU, 0x7ffbff3fU, 0xffffab5bU, 0xe7fff35bU}, {0x7ffbff3fU, 0xffdffbaeU, 0xe7fff35bU, 0xffffab5bU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 2, 12, 5, 3, 3, 0xbfdefeefU, 0xbffffeffU, 0xfffefffbU, 0xfbfbdffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {12U, 12U, 12U, 12U}, {3U, 3U, 3U, 3U}, {0xbfdefeefU, 0xbffffeffU, 0xfffefffbU, 0xfbfbdffeU}, {0xbffffeffU, 0xbfdefeefU, 0xfbfbdffeU, 0xfffefffbU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 6, 3, 3, 5, 3, 0xfffffff3U, 0xf7bbffffU, 0xfafff9fbU, 0xff77fddfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x80000000U, {3U, 3U, 3U, 3U}, {5U, 5U, 5U, 5U}, {0xfffffff3U, 0xf7bbffffU, 0xfafff9fbU, 0xff77fddfU}, {0xf7bbffffU, 0xfffffff3U, 0xff77fddfU, 0xfafff9fbU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 6, 19, 1, 7, 3, 0xfdffefffU, 0x5ffbdffdU, 0xefbbfff3U, 0xff7e7bb7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {19U, 19U, 19U, 19U}, {7U, 7U, 7U, 7U}, {0xfdffefffU, 0x5ffbdffdU, 0xefbbfff3U, 0xff7e7bb7U}, {0x5ffbdffdU, 0xfdffefffU, 0xff7e7bb7U, 0xefbbfff3U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 8, 11, 1, 5, 3, 0x7eafebffU, 0x7fdfdefdU, 0xb7d7bff5U, 0xf7bb7ff7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x40000000U, {11U, 11U, 11U, 11U}, {5U, 5U, 5U, 5U}, {0x7eafebffU, 0x7fdfdefdU, 0xb7d7bff5U, 0xf7bb7ff7U}, {0x7fdfdefdU, 0x7eafebffU, 0xf7bb7ff7U, 0xb7d7bff5U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 1279, 6, 14, 1, 3, 7, 0xffffbffdU, 0xfffbffdfU, 0xffffffffU, 0xfedffde5U, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {14U, 14U, 14U, 14U}, {3U, 3U, 3U, 3U}, {0xffffbffdU, 0xfffbffdfU, 0xffffffffU, 0xfedffde5U}, {0xfffbffdfU, 0xffffbffdU, 0xfedffde5U, 0xffffffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 1279, 3, 5, 3, 22, 5, 0xcffdffdfU, 0xffff6effU, 0xfff5fff7U, 0xefffbfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x00000000U, {5U, 5U, 5U, 5U}, {22U, 22U, 22U, 22U}, {0xcffdffdfU, 0xffff6effU, 0xfff5fff7U, 0xefffbfffU}, {0xffff6effU, 0xcffdffdfU, 0xefffbfffU, 0xfff5fff7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 

// 19937.csv
{ 19937, 122, 18, 1, 11, 1, 0xdfffffefU, 0xddfecb7fU, 0xbffaffffU, 0xbffffff6U, 0x00000001U, 0x00000000U, 0x00000000U, 0x13c9e684U, {18U, 18U, 18U, 18U}, {11U, 11U, 11U, 11U}, {0xdfffffefU, 0xddfecb7fU, 0xbffaffffU, 0xbffffff6U}, {0xddfecb7fU, 0xdfffffefU, 0xbffffff6U, 0xbffaffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 73, 11, 3, 4, 1, 0xefefffffU, 0xef7dfffbU, 0xbcfffeffU, 0xfdffffc7U, 0x00000001U, 0x00000000U, 0x00000000U, 0xfbdf63e3U, {11U, 11U, 11U, 11U}, {4U, 4U, 4U, 4U}, {0xefefffffU, 0xef7dfffbU, 0xbcfffeffU, 0xfdffffc7U}, {0xef7dfffbU, 0xefefffffU, 0xfdffffc7U, 0xbcfffeffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 60, 10, 3, 3, 1, 0xeffdbfffU, 0xdfffffffU, 0xefffbfefU, 0xffef77ffU, 0x00000001U, 0x00000000U, 0x00000000U, 0xdc0d7984U, {10U, 10U, 10U, 10U}, {3U, 3U, 3U, 3U}, {0xeffdbfffU, 0xdfffffffU, 0xefffbfefU, 0xffef77ffU}, {0xdfffffffU, 0xeffdbfffU, 0xffef77ffU, 0xefffbfefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 45, 14, 3, 5, 5, 0xaefefbbdU, 0xfeffffffU, 0xd7edffffU, 0xf7fdffbeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x0116914cU, {14U, 14U, 14U, 14U}, {5U, 5U, 5U, 5U}, {0xaefefbbdU, 0xfeffffffU, 0xd7edffffU, 0xf7fdffbeU}, {0xfeffffffU, 0xaefefbbdU, 0xf7fdffbeU, 0xd7edffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 19937, 135, 18, 1, 1, 1, 0xffffdffeU, 0xfffefffbU, 0xbfbfffffU, 0xfffffdffU, 0x00000001U, 0x00000000U, 0x40000000U, 0x978e01e2U, {18U, 18U, 18U, 18U}, {1U, 1U, 1U, 1U}, {0xffffdffeU, 0xfffefffbU, 0xbfbfffffU, 0xfffffdffU}, {0xfffefffbU, 0xffffdffeU, 0xfffffdffU, 0xbfbfffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 120, 18, 1, 1, 1, 0xffefef5dU, 0xff25ff7dU, 0xeebfefeeU, 0xfabfdfffU, 0x00000001U, 0x00000000U, 0x02000000U, 0xc9bbd290U, {18U, 18U, 18U, 18U}, {1U, 1U, 1U, 1U}, {0xffefef5dU, 0xff25ff7dU, 0xeebfefeeU, 0xfabfdfffU}, {0xff25ff7dU, 0xffefef5dU, 0xfabfdfffU, 0xeebfefeeU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 33, 14, 3, 5, 1, 0xfdff7ffdU, 0xfbffee71U, 0xfb3ff7dfU, 0xdeffffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x8f2cf308U, {14U, 14U, 14U, 14U}, {5U, 5U, 5U, 5U}, {0xfdff7ffdU, 0xfbffee71U, 0xfb3ff7dfU, 0xdeffffffU}, {0xfbffee71U, 0xfdff7ffdU, 0xdeffffffU, 0xfb3ff7dfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 122, 12, 3, 3, 5, 0xfef73bffU, 0x65eb7ff7U, 0x76ffffffU, 0xbfdeffebU, 0x00000001U, 0x00000000U, 0x40000000U, 0x2bbdd900U, {12U, 12U, 12U, 12U}, {3U, 3U, 3U, 3U}, {0xfef73bffU, 0x65eb7ff7U, 0x76ffffffU, 0xbfdeffebU}, {0x65eb7ff7U, 0xfef73bffU, 0xbfdeffebU, 0x76ffffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 19937, 64, 10, 3, 13, 1, 0xfff8bfffU, 0xffffffffU, 0x5fffffbfU, 0xef7ff21eU, 0x00000001U, 0x00000000U, 0x00000000U, 0xdb186980U, {10U, 10U, 10U, 10U}, {13U, 13U, 13U, 13U}, {0xfff8bfffU, 0xffffffffU, 0x5fffffbfU, 0xef7ff21eU}, {0xffffffffU, 0xfff8bfffU, 0xef7ff21eU, 0x5fffffbfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 29, 21, 1, 2, 1, 0x5fffffdfU, 0xffffcfffU, 0xbdffdefeU, 0xfdfdffbfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x21a2acc8U, {21U, 21U, 21U, 21U}, {2U, 2U, 2U, 2U}, {0x5fffffdfU, 0xffffcfffU, 0xbdffdefeU, 0xfdfdffbfU}, {0xffffcfffU, 0x5fffffdfU, 0xfdfdffbfU, 0xbdffdefeU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 128, 11, 3, 1, 5, 0x7dbebfffU, 0x7ffffffeU, 0xded6ffefU, 0xfefeff6eU, 0x00000001U, 0x00000000U, 0x00000000U, 0x42530085U, {11U, 11U, 11U, 11U}, {1U, 1U, 1U, 1U}, {0x7dbebfffU, 0x7ffffffeU, 0xded6ffefU, 0xfefeff6eU}, {0x7ffffffeU, 0x7dbebfffU, 0xfefeff6eU, 0xded6ffefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 19937, 106, 19, 1, 9, 1, 0xffeeffdfU, 0xffffdfffU, 0xbd6ff9ffU, 0xfdafebffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x47a2caf4U, {19U, 19U, 19U, 19U}, {9U, 9U, 9U, 9U}, {0xffeeffdfU, 0xffffdfffU, 0xbd6ff9ffU, 0xfdafebffU}, {0xffffdfffU, 0xffeeffdfU, 0xfdafebffU, 0xbd6ff9ffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 75, 9, 3, 4, 3, 0xfffeffffU, 0xffeff7dfU, 0xbbbffeefU, 0xbfbffeffU, 0x00000001U, 0x00000000U, 0x80000000U, 0x0b6f3e5cU, {9U, 9U, 9U, 9U}, {4U, 4U, 4U, 4U}, {0xfffeffffU, 0xffeff7dfU, 0xbbbffeefU, 0xbfbffeffU}, {0xffeff7dfU, 0xfffeffffU, 0xbfbffeffU, 0xbbbffeefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 19937, 85, 10, 3, 5, 5, 0xbdfffbefU, 0xefedffefU, 0xaff9f7feU, 0xbefbfffdU, 0x00000001U, 0x00000000U, 0x00000000U, 0x8eddf8b2U, {10U, 10U, 10U, 10U}, {5U, 5U, 5U, 5U}, {0xbdfffbefU, 0xefedffefU, 0xaff9f7feU, 0xbefbfffdU}, {0xefedffefU, 0xbdfffbefU, 0xbefbfffdU, 0xaff9f7feU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 19937, 27, 21, 1, 12, 1, 0xbf7dffdfU, 0xc97dff7bU, 0xf7fdfbfdU, 0xf7bffbafU, 0x00000001U, 0x00000000U, 0x00000000U, 0x539ef018U, {21U, 21U, 21U, 21U}, {12U, 12U, 12U, 12U}, {0xbf7dffdfU, 0xc97dff7bU, 0xf7fdfbfdU, 0xf7bffbafU}, {0xc97dff7bU, 0xbf7dffdfU, 0xf7bffbafU, 0xf7fdfbfdU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 12, 13, 3, 3, 1, 0xffffffbbU, 0xfffff7abU, 0xfdfff7c2U, 0xbdff9efbU, 0x00000001U, 0x00000000U, 0x00000000U, 0xa631116aU, {13U, 13U, 13U, 13U}, {3U, 3U, 3U, 3U}, {0xffffffbbU, 0xfffff7abU, 0xfdfff7c2U, 0xbdff9efbU}, {0xfffff7abU, 0xffffffbbU, 0xbdff9efbU, 0xfdfff7c2U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 92, 9, 3, 5, 3, 0xcfffbbffU, 0xfffffff7U, 0xfff7ffefU, 0xff9fffdfU, 0x00000001U, 0x00000000U, 0x00000000U, 0xfb62abccU, {9U, 9U, 9U, 9U}, {5U, 5U, 5U, 5U}, {0xcfffbbffU, 0xfffffff7U, 0xfff7ffefU, 0xff9fffdfU}, {0xfffffff7U, 0xcfffbbffU, 0xff9fffdfU, 0xfff7ffefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 19937, 124, 13, 3, 11, 1, 0xffbff7efU, 0x7b7e7fcdU, 0xfddaff7fU, 0xb7dbfdfcU, 0x00000001U, 0x00000000U, 0x00000000U, 0x02889855U, {13U, 13U, 13U, 13U}, {11U, 11U, 11U, 11U}, {0xffbff7efU, 0x7b7e7fcdU, 0xfddaff7fU, 0xb7dbfdfcU}, {0x7b7e7fcdU, 0xffbff7efU, 0xb7dbfdfcU, 0xfddaff7fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 112, 20, 1, 15, 1, 0xffbf7efeU, 0xfabf7fffU, 0xf6ff7fefU, 0xf74f6f3fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x921dce22U, {20U, 20U, 20U, 20U}, {15U, 15U, 15U, 15U}, {0xffbf7efeU, 0xfabf7fffU, 0xf6ff7fefU, 0xf74f6f3fU}, {0xfabf7fffU, 0xffbf7efeU, 0xf74f6f3fU, 0xf6ff7fefU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 130, 9, 3, 13, 1, 0xdfbf1ffdU, 0xf7feefddU, 0xfffbffffU, 0x96ffffdfU, 0x00000001U, 0x00000000U, 0x00000000U, 0xda18b200U, {9U, 9U, 9U, 9U}, {13U, 13U, 13U, 13U}, {0xdfbf1ffdU, 0xf7feefddU, 0xfffbffffU, 0x96ffffdfU}, {0xf7feefddU, 0xdfbf1ffdU, 0x96ffffdfU, 0xfffbffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 125, 11, 3, 16, 3, 0xbfdfffffU, 0x7fffbe7fU, 0xbbfffff7U, 0xf7fdddffU, 0x00000001U, 0x00000000U, 0x40000000U, 0x63cd4800U, {11U, 11U, 11U, 11U}, {16U, 16U, 16U, 16U}, {0xbfdfffffU, 0x7fffbe7fU, 0xbbfffff7U, 0xf7fdddffU}, {0x7fffbe7fU, 0xbfdfffffU, 0xf7fdddffU, 0xbbfffff7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 19937, 4, 20, 1, 1, 1, 0xfffef7f9U, 0xffdff7fbU, 0xcbfff7feU, 0xedfefffdU, 0x00000001U, 0x00000000U, 0x80000000U, 0xd71cbdcaU, {20U, 20U, 20U, 20U}, {1U, 1U, 1U, 1U}, {0xfffef7f9U, 0xffdff7fbU, 0xcbfff7feU, 0xedfefffdU}, {0xffdff7fbU, 0xfffef7f9U, 0xedfefffdU, 0xcbfff7feU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 123, 16, 3, 3, 1, 0xbd7feffdU, 0xbfffffffU, 0xfffcf6ffU, 0x5ffcffebU, 0x00000001U, 0x00000000U, 0x00000000U, 0x09d68a18U, {16U, 16U, 16U, 16U}, {3U, 3U, 3U, 3U}, {0xbd7feffdU, 0xbfffffffU, 0xfffcf6ffU, 0x5ffcffebU}, {0xbfffffffU, 0xbd7feffdU, 0x5ffcffebU, 0xfffcf6ffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 97, 14, 1, 11, 1, 0xefdfceffU, 0xeddffdffU, 0xefdfef7fU, 0xffb7ffdfU, 0x00000001U, 0x00000000U, 0x00000000U, 0xc491f160U, {14U, 14U, 14U, 14U}, {11U, 11U, 11U, 11U}, {0xefdfceffU, 0xeddffdffU, 0xefdfef7fU, 0xffb7ffdfU}, {0xeddffdffU, 0xefdfceffU, 0xffb7ffdfU, 0xefdfef7fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 2, 13, 3, 11, 1, 0xfdffbffdU, 0xfff3b33fU, 0xfff7ff7eU, 0xffdfef7fU, 0x00000001U, 0x00000000U, 0x80000000U, 0xad42c1d1U, {13U, 13U, 13U, 13U}, {11U, 11U, 11U, 11U}, {0xfdffbffdU, 0xfff3b33fU, 0xfff7ff7eU, 0xffdfef7fU}, {0xfff3b33fU, 0xfdffbffdU, 0xffdfef7fU, 0xfff7ff7eU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 153, 11, 3, 4, 3, 0xffebf7fdU, 0xfffdffffU, 0xffffffefU, 0xfffbdfb7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x7f72515eU, {11U, 11U, 11U, 11U}, {4U, 4U, 4U, 4U}, {0xffebf7fdU, 0xfffdffffU, 0xffffffefU, 0xfffbdfb7U}, {0xfffdffffU, 0xffebf7fdU, 0xfffbdfb7U, 0xffffffefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 19937, 64, 7, 7, 7, 1, 0xfbbefeefU, 0xffffffc7U, 0xfbffed57U, 0xfafffddfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x7094862aU, {7U, 7U, 7U, 7U}, {7U, 7U, 7U, 7U}, {0xfbbefeefU, 0xffffffc7U, 0xfbffed57U, 0xfafffddfU}, {0xffffffc7U, 0xfbbefeefU, 0xfafffddfU, 0xfbffed57U}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 150, 7, 3, 3, 1, 0xffff7fefU, 0xffbf5affU, 0xa6ff7fbdU, 0xbffffbffU, 0x00000001U, 0x00000000U, 0x40000000U, 0x5ee9dc90U, {7U, 7U, 7U, 7U}, {3U, 3U, 3U, 3U}, {0xffff7fefU, 0xffbf5affU, 0xa6ff7fbdU, 0xbffffbffU}, {0xffbf5affU, 0xffff7fefU, 0xbffffbffU, 0xa6ff7fbdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 150, 16, 3, 3, 1, 0xfdbffaedU, 0xffc7f69fU, 0x67f7ffbeU, 0xbf7bbf7eU, 0x00000001U, 0x00000000U, 0x00000000U, 0xb6a0c2e9U, {16U, 16U, 16U, 16U}, {3U, 3U, 3U, 3U}, {0xfdbffaedU, 0xffc7f69fU, 0x67f7ffbeU, 0xbf7bbf7eU}, {0xffc7f69fU, 0xfdbffaedU, 0xbf7bbf7eU, 0x67f7ffbeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 110, 17, 3, 9, 3, 0xbffffffbU, 0x7b7fffffU, 0xffffdf5fU, 0xfbffdeffU, 0x00000001U, 0x00000000U, 0x20000000U, 0x4bf4dbf0U, {17U, 17U, 17U, 17U}, {9U, 9U, 9U, 9U}, {0xbffffffbU, 0x7b7fffffU, 0xffffdf5fU, 0xfbffdeffU}, {0x7b7fffffU, 0xbffffffbU, 0xfbffdeffU, 0xffffdf5fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 19937, 22, 5, 3, 7, 1, 0xbb5fffffU, 0xbe7fdfffU, 0xfe7fffdfU, 0x6ffaffffU, 0x00000001U, 0x00000000U, 0x80000000U, 0x43471de2U, {5U, 5U, 5U, 5U}, {7U, 7U, 7U, 7U}, {0xbb5fffffU, 0xbe7fdfffU, 0xfe7fffdfU, 0x6ffaffffU}, {0xbe7fdfffU, 0xbb5fffffU, 0x6ffaffffU, 0xfe7fffdfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 19937, 104, 9, 7, 13, 1, 0xbfefefebU, 0x77f7efbfU, 0xf5ffff7eU, 0xfffffffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0xc4cda29cU, {9U, 9U, 9U, 9U}, {13U, 13U, 13U, 13U}, {0xbfefefebU, 0x77f7efbfU, 0xf5ffff7eU, 0xfffffffeU}, {0x77f7efbfU, 0xbfefefebU, 0xfffffffeU, 0xf5ffff7eU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 

// 2281.csv
{ 2281, 12, 19, 1, 5, 1, 0xbff7ffbfU, 0xfdfffffeU, 0xf7ffef7fU, 0xf2f7cbbfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x41dfa600U, {19U, 19U, 19U, 19U}, {5U, 5U, 5U, 5U}, {0xbff7ffbfU, 0xfdfffffeU, 0xf7ffef7fU, 0xf2f7cbbfU}, {0xfdfffffeU, 0xbff7ffbfU, 0xf2f7cbbfU, 0xf7ffef7fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 12, 10, 3, 5, 1, 0xfbdffff7U, 0xcfbffffeU, 0xfdfffffbU, 0xbcdffffbU, 0x00000001U, 0x00000000U, 0x00000000U, 0x92730900U, {10U, 10U, 10U, 10U}, {5U, 5U, 5U, 5U}, {0xfbdffff7U, 0xcfbffffeU, 0xfdfffffbU, 0xbcdffffbU}, {0xcfbffffeU, 0xfbdffff7U, 0xbcdffffbU, 0xfdfffffbU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 7, 9, 3, 4, 1, 0xdfffef7bU, 0xffefffffU, 0xfcfeffffU, 0xb6ff77cfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x907c9c00U, {9U, 9U, 9U, 9U}, {4U, 4U, 4U, 4U}, {0xdfffef7bU, 0xffefffffU, 0xfcfeffffU, 0xb6ff77cfU}, {0xffefffffU, 0xdfffef7bU, 0xb6ff77cfU, 0xfcfeffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 8, 10, 3, 7, 1, 0xdbffedffU, 0xfffbfffdU, 0xf7ffffdfU, 0xffff7ff7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x0a826800U, {10U, 10U, 10U, 10U}, {7U, 7U, 7U, 7U}, {0xdbffedffU, 0xfffbfffdU, 0xf7ffffdfU, 0xffff7ff7U}, {0xfffbfffdU, 0xdbffedffU, 0xffff7ff7U, 0xf7ffffdfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 5, 10, 3, 9, 5, 0x3f7ffbffU, 0xfffffff9U, 0xbffffbfeU, 0x9ffff7ffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x92e7de00U, {10U, 10U, 10U, 10U}, {9U, 9U, 9U, 9U}, {0x3f7ffbffU, 0xfffffff9U, 0xbffffbfeU, 0x9ffff7ffU}, {0xfffffff9U, 0x3f7ffbffU, 0x9ffff7ffU, 0xbffffbfeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 2281, 10, 10, 3, 9, 3, 0xff9ffff7U, 0xfdffffffU, 0xbf3f7fffU, 0xfbffff7cU, 0x00000001U, 0x00000000U, 0x00000000U, 0xc66e2000U, {10U, 10U, 10U, 10U}, {9U, 9U, 9U, 9U}, {0xff9ffff7U, 0xfdffffffU, 0xbf3f7fffU, 0xfbffff7cU}, {0xfdffffffU, 0xff9ffff7U, 0xfbffff7cU, 0xbf3f7fffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 2281, 11, 8, 3, 5, 5, 0xff6ffba9U, 0xfffff7f7U, 0xffdfdfffU, 0xf2ffffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x1b40e800U, {8U, 8U, 8U, 8U}, {5U, 5U, 5U, 5U}, {0xff6ffba9U, 0xfffff7f7U, 0xffdfdfffU, 0xf2ffffffU}, {0xfffff7f7U, 0xff6ffba9U, 0xf2ffffffU, 0xffdfdfffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 2281, 3, 14, 3, 11, 5, 0x7bffffdfU, 0x3ffb7fdbU, 0x3f777ffbU, 0xfffefdbbU, 0x00000001U, 0x00000000U, 0x00000000U, 0x3530d200U, {14U, 14U, 14U, 14U}, {11U, 11U, 11U, 11U}, {0x7bffffdfU, 0x3ffb7fdbU, 0x3f777ffbU, 0xfffefdbbU}, {0x3ffb7fdbU, 0x7bffffdfU, 0xfffefdbbU, 0x3f777ffbU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 2281, 11, 14, 7, 5, 1, 0xf7bffff9U, 0x6efff7ffU, 0xfbfbffdfU, 0xfff8ffbfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x48acb400U, {14U, 14U, 14U, 14U}, {5U, 5U, 5U, 5U}, {0xf7bffff9U, 0x6efff7ffU, 0xfbfbffdfU, 0xfff8ffbfU}, {0x6efff7ffU, 0xf7bffff9U, 0xfff8ffbfU, 0xfbfbffdfU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 2, 20, 1, 5, 3, 0xffffffbfU, 0x7dffdfebU, 0xdf7fffdfU, 0xfeff69eeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x32455000U, {20U, 20U, 20U, 20U}, {5U, 5U, 5U, 5U}, {0xffffffbfU, 0x7dffdfebU, 0xdf7fffdfU, 0xfeff69eeU}, {0x7dffdfebU, 0xffffffbfU, 0xfeff69eeU, 0xdf7fffdfU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 2281, 5, 10, 3, 15, 5, 0xfff3ecfdU, 0xffffff77U, 0x9f9fadffU, 0xb9d3ebddU, 0x00000001U, 0x00000000U, 0x00000000U, 0xb05f9800U, {10U, 10U, 10U, 10U}, {15U, 15U, 15U, 15U}, {0xfff3ecfdU, 0xffffff77U, 0x9f9fadffU, 0xb9d3ebddU}, {0xffffff77U, 0xfff3ecfdU, 0xb9d3ebddU, 0x9f9fadffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 2281, 15, 13, 1, 4, 1, 0xffffff1bU, 0xffbff77bU, 0xbfbffffeU, 0xfffbfaf5U, 0x00000001U, 0x00000000U, 0x00000000U, 0x27b07600U, {13U, 13U, 13U, 13U}, {4U, 4U, 4U, 4U}, {0xffffff1bU, 0xffbff77bU, 0xbfbffffeU, 0xfffbfaf5U}, {0xffbff77bU, 0xffffff1bU, 0xfffbfaf5U, 0xbfbffffeU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 5, 5, 3, 6, 1, 0xdfff7dffU, 0xff79fffeU, 0xeff77ffdU, 0xeffb75dfU, 0x00000001U, 0x00000000U, 0x00000000U, 0xc22bf800U, {5U, 5U, 5U, 5U}, {6U, 6U, 6U, 6U}, {0xdfff7dffU, 0xff79fffeU, 0xeff77ffdU, 0xeffb75dfU}, {0xff79fffeU, 0xdfff7dffU, 0xeffb75dfU, 0xeff77ffdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 16, 18, 5, 1, 1, 0x7ffffbffU, 0xfffff7bfU, 0xdf7e6fffU, 0xfff7cfcfU, 0x00000001U, 0x00000000U, 0x00000000U, 0x3d350a80U, {18U, 18U, 18U, 18U}, {1U, 1U, 1U, 1U}, {0x7ffffbffU, 0xfffff7bfU, 0xdf7e6fffU, 0xfff7cfcfU}, {0xfffff7bfU, 0x7ffffbffU, 0xfff7cfcfU, 0xdf7e6fffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 15, 19, 3, 14, 1, 0xffffdbbfU, 0xcbf7ffffU, 0xfffebbfdU, 0xbefebfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x147a0200U, {19U, 19U, 19U, 19U}, {14U, 14U, 14U, 14U}, {0xffffdbbfU, 0xcbf7ffffU, 0xfffebbfdU, 0xbefebfffU}, {0xcbf7ffffU, 0xffffdbbfU, 0xbefebfffU, 0xfffebbfdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 9, 10, 7, 13, 1, 0xdfffeeefU, 0xfffffddfU, 0xfbfbfe5fU, 0xcffbfdefU, 0x00000001U, 0x00000000U, 0x00000000U, 0xdb3cce00U, {10U, 10U, 10U, 10U}, {13U, 13U, 13U, 13U}, {0xdfffeeefU, 0xfffffddfU, 0xfbfbfe5fU, 0xcffbfdefU}, {0xfffffddfU, 0xdfffeeefU, 0xcffbfdefU, 0xfbfbfe5fU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 4, 14, 3, 13, 1, 0xd7ffffdfU, 0xbfdfffffU, 0xfbfffdfeU, 0x5ffbffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x011ecc00U, {14U, 14U, 14U, 14U}, {13U, 13U, 13U, 13U}, {0xd7ffffdfU, 0xbfdfffffU, 0xfbfffdfeU, 0x5ffbffffU}, {0xbfdfffffU, 0xd7ffffdfU, 0x5ffbffffU, 0xfbfffdfeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 1, 11, 3, 24, 1, 0xf3fbbfffU, 0xbffffffbU, 0xdf7fd7bdU, 0xff7edffdU, 0x00000001U, 0x00000000U, 0x00000000U, 0x07aee200U, {11U, 11U, 11U, 11U}, {24U, 24U, 24U, 24U}, {0xf3fbbfffU, 0xbffffffbU, 0xdf7fd7bdU, 0xff7edffdU}, {0xbffffffbU, 0xf3fbbfffU, 0xff7edffdU, 0xdf7fd7bdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 6, 9, 7, 21, 1, 0xbeddbdffU, 0xfbffbd7fU, 0xf95b7fffU, 0x7fefbfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x197e2080U, {9U, 9U, 9U, 9U}, {21U, 21U, 21U, 21U}, {0xbeddbdffU, 0xfbffbd7fU, 0xf95b7fffU, 0x7fefbfffU}, {0xfbffbd7fU, 0xbeddbdffU, 0x7fefbfffU, 0xf95b7fffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 1, 9, 3, 12, 1, 0xeff7fcfaU, 0xf3ee7fefU, 0xbdb47dffU, 0xdbf6dfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x2bbd0a00U, {9U, 9U, 9U, 9U}, {12U, 12U, 12U, 12U}, {0xeff7fcfaU, 0xf3ee7fefU, 0xbdb47dffU, 0xdbf6dfffU}, {0xf3ee7fefU, 0xeff7fcfaU, 0xdbf6dfffU, 0xbdb47dffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 1, 18, 5, 7, 3, 0xefffffe7U, 0xffffdfdfU, 0xffffffffU, 0x5fff4affU, 0x00000001U, 0x00000000U, 0x00000000U, 0x8a50e100U, {18U, 18U, 18U, 18U}, {7U, 7U, 7U, 7U}, {0xefffffe7U, 0xffffdfdfU, 0xffffffffU, 0x5fff4affU}, {0xffffdfdfU, 0xefffffe7U, 0x5fff4affU, 0xffffffffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 2281, 11, 19, 7, 12, 3, 0xf7bfffffU, 0xeddfffffU, 0xf77ffd7fU, 0xeff7ffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x502bdc20U, {19U, 19U, 19U, 19U}, {12U, 12U, 12U, 12U}, {0xf7bfffffU, 0xeddfffffU, 0xf77ffd7fU, 0xeff7ffffU}, {0xeddfffffU, 0xf7bfffffU, 0xeff7ffffU, 0xf77ffd7fU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 2281, 10, 19, 3, 17, 1, 0xeffbdfffU, 0xfffdbeedU, 0xfefd3fbfU, 0xff5beffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0xdedc0400U, {19U, 19U, 19U, 19U}, {17U, 17U, 17U, 17U}, {0xeffbdfffU, 0xfffdbeedU, 0xfefd3fbfU, 0xff5beffeU}, {0xfffdbeedU, 0xeffbdfffU, 0xff5beffeU, 0xfefd3fbfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 11, 17, 7, 20, 1, 0xbdffddfdU, 0xdfdffffcU, 0xbffeffd6U, 0xff7fffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x3923e800U, {17U, 17U, 17U, 17U}, {20U, 20U, 20U, 20U}, {0xbdffddfdU, 0xdfdffffcU, 0xbffeffd6U, 0xff7fffffU}, {0xdfdffffcU, 0xbdffddfdU, 0xff7fffffU, 0xbffeffd6U}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 9, 21, 3, 20, 5, 0xffffffdfU, 0xdfffffffU, 0xff6ffffaU, 0xffffdfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x8a9b2080U, {21U, 21U, 21U, 21U}, {20U, 20U, 20U, 20U}, {0xffffffdfU, 0xdfffffffU, 0xff6ffffaU, 0xffffdfffU}, {0xdfffffffU, 0xffffffdfU, 0xffffdfffU, 0xff6ffffaU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 2281, 5, 4, 7, 19, 1, 0xffbfbfffU, 0xffffbfffU, 0xfff7ceeeU, 0xffffefffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x859df000U, {4U, 4U, 4U, 4U}, {19U, 19U, 19U, 19U}, {0xffbfbfffU, 0xffffbfffU, 0xfff7ceeeU, 0xffffefffU}, {0xffffbfffU, 0xffbfbfffU, 0xffffefffU, 0xfff7ceeeU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 3, 2, 7, 1, 1, 0xfffefbefU, 0x7ffffbf7U, 0xffffffffU, 0x6fff7fffU, 0x00000001U, 0x00000000U, 0x00000000U, 0xcc1df840U, {2U, 2U, 2U, 2U}, {1U, 1U, 1U, 1U}, {0xfffefbefU, 0x7ffffbf7U, 0xffffffffU, 0x6fff7fffU}, {0x7ffffbf7U, 0xfffefbefU, 0x6fff7fffU, 0xffffffffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 14, 20, 7, 9, 1, 0x7bffc3f9U, 0xddfff2fbU, 0xddfbf6fbU, 0x77dfdfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0xf6dae320U, {20U, 20U, 20U, 20U}, {9U, 9U, 9U, 9U}, {0x7bffc3f9U, 0xddfff2fbU, 0xddfbf6fbU, 0x77dfdfffU}, {0xddfff2fbU, 0x7bffc3f9U, 0x77dfdfffU, 0xddfbf6fbU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 3, 2, 3, 19, 1, 0xbbfffcffU, 0xf7bffcdfU, 0xe39ffffdU, 0xf7bebfffU, 0x00000001U, 0x00000000U, 0x00000000U, 0xd324a000U, {2U, 2U, 2U, 2U}, {19U, 19U, 19U, 19U}, {0xbbfffcffU, 0xf7bffcdfU, 0xe39ffffdU, 0xf7bebfffU}, {0xf7bffcdfU, 0xbbfffcffU, 0xf7bebfffU, 0xe39ffffdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 2, 9, 7, 17, 1, 0xffffffffU, 0x7fbff9dfU, 0x7f6fefcfU, 0xefffffdeU, 0x00000001U, 0x00000000U, 0x00000000U, 0xcb8f6a00U, {9U, 9U, 9U, 9U}, {17U, 17U, 17U, 17U}, {0xffffffffU, 0x7fbff9dfU, 0x7f6fefcfU, 0xefffffdeU}, {0x7fbff9dfU, 0xffffffffU, 0xefffffdeU, 0x7f6fefcfU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 2281, 15, 12, 1, 1, 7, 0xfdffffffU, 0xffffdef7U, 0xff3fffffU, 0xf37776f7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x16537800U, {12U, 12U, 12U, 12U}, {1U, 1U, 1U, 1U}, {0xfdffffffU, 0xffffdef7U, 0xff3fffffU, 0xf37776f7U}, {0xffffdef7U, 0xfdffffffU, 0xf37776f7U, 0xff3fffffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 2281, 9, 11, 5, 6, 3, 0xe7fffba3U, 0xffdf7fdfU, 0xefbf7fffU, 0xfd7cf7bfU, 0x00000001U, 0x00000000U, 0x00000000U, 0xc4cbc800U, {11U, 11U, 11U, 11U}, {6U, 6U, 6U, 6U}, {0xe7fffba3U, 0xffdf7fdfU, 0xefbf7fffU, 0xfd7cf7bfU}, {0xffdf7fdfU, 0xe7fffba3U, 0xfd7cf7bfU, 0xefbf7fffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 

// 4253.csv
{ 4253, 17, 20, 1, 7, 1, 0x9f7bffffU, 0x9fffff5fU, 0x3efffffbU, 0xfffff7bbU, 0xa8000001U, 0xaf5390a3U, 0xb740b3f8U, 0x6c11486dU, {20U, 20U, 20U, 20U}, {7U, 7U, 7U, 7U}, {0x9f7bffffU, 0x9fffff5fU, 0x3efffffbU, 0xfffff7bbU}, {0x9fffff5fU, 0x9f7bffffU, 0xfffff7bbU, 0x3efffffbU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 25, 10, 3, 1, 3, 0xdaeeedf7U, 0xfff4eff7U, 0xffffebfeU, 0xeff7febfU, 0xc0000001U, 0x078d2186U, 0xe690a4a9U, 0x7a5efbe5U, {10U, 10U, 10U, 10U}, {1U, 1U, 1U, 1U}, {0xdaeeedf7U, 0xfff4eff7U, 0xffffebfeU, 0xeff7febfU}, {0xfff4eff7U, 0xdaeeedf7U, 0xeff7febfU, 0xffffebfeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 15, 25, 1, 2, 1, 0xdffff3ffU, 0xffdebf7fU, 0xffdfbdf7U, 0xfbffffffU, 0x20000001U, 0x8b4ecd15U, 0xb3480fc1U, 0x35ab1967U, {25U, 25U, 25U, 25U}, {2U, 2U, 2U, 2U}, {0xdffff3ffU, 0xffdebf7fU, 0xffdfbdf7U, 0xfbffffffU}, {0xffdebf7fU, 0xdffff3ffU, 0xfbffffffU, 0xffdfbdf7U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 28, 8, 3, 3, 5, 0xafffdffbU, 0xffffdfbfU, 0xfbffdf77U, 0x7d7cfef7U, 0x20000001U, 0x8ed2d92aU, 0xefc5d057U, 0xa6e061d3U, {8U, 8U, 8U, 8U}, {3U, 3U, 3U, 3U}, {0xafffdffbU, 0xffffdfbfU, 0xfbffdf77U, 0x7d7cfef7U}, {0xffffdfbfU, 0xafffdffbU, 0x7d7cfef7U, 0xfbffdf77U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 4253, 10, 17, 3, 13, 3, 0xbffdefffU, 0xffaffeffU, 0x9f3ebfceU, 0xbf9fffdfU, 0xa0000001U, 0xd3bdda16U, 0x4140113fU, 0xd458c634U, {17U, 17U, 17U, 17U}, {13U, 13U, 13U, 13U}, {0xbffdefffU, 0xffaffeffU, 0x9f3ebfceU, 0xbf9fffdfU}, {0xffaffeffU, 0xbffdefffU, 0xbf9fffdfU, 0x9f3ebfceU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 8, 7, 3, 19, 1, 0xbffffffbU, 0x6f9ffbffU, 0x7f9fe7f7U, 0xbfeffffbU, 0x40000001U, 0x4e4b2758U, 0xb998e05aU, 0xc4f1b268U, {7U, 7U, 7U, 7U}, {19U, 19U, 19U, 19U}, {0xbffffffbU, 0x6f9ffbffU, 0x7f9fe7f7U, 0xbfeffffbU}, {0x6f9ffbffU, 0xbffffffbU, 0xbfeffffbU, 0x7f9fe7f7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 13, 18, 1, 21, 1, 0xe777f7fcU, 0xf7d7bf7dU, 0xedff7f5fU, 0xffbfff7fU, 0x40000001U, 0x7df3317aU, 0xbbfeb4c7U, 0x13a23b1dU, {18U, 18U, 18U, 18U}, {21U, 21U, 21U, 21U}, {0xe777f7fcU, 0xf7d7bf7dU, 0xedff7f5fU, 0xffbfff7fU}, {0xf7d7bf7dU, 0xe777f7fcU, 0xffbfff7fU, 0xedff7f5fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 14, 6, 3, 3, 5, 0x7fdff3fbU, 0x57dfffdbU, 0xfe9ffddfU, 0xfefdae7fU, 0x42000001U, 0xeb34cfceU, 0x3eab62b1U, 0x95b6f6ebU, {6U, 6U, 6U, 6U}, {3U, 3U, 3U, 3U}, {0x7fdff3fbU, 0x57dfffdbU, 0xfe9ffddfU, 0xfefdae7fU}, {0x57dfffdbU, 0x7fdff3fbU, 0xfefdae7fU, 0xfe9ffddfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 4253, 23, 11, 7, 6, 1, 0xffbfbe7fU, 0xffbfdf6dU, 0xbfbffdfdU, 0xfffffffdU, 0x60000001U, 0x045bc889U, 0x7b26b276U, 0xf4e713f6U, {11U, 11U, 11U, 11U}, {6U, 6U, 6U, 6U}, {0xffbfbe7fU, 0xffbfdf6dU, 0xbfbffdfdU, 0xfffffffdU}, {0xffbfdf6dU, 0xffbfbe7fU, 0xfffffffdU, 0xbfbffdfdU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 16, 16, 3, 11, 3, 0xfdffff4fU, 0xfff7fbffU, 0xee7ffdffU, 0xdfbbfdffU, 0xc0000001U, 0xeb6e2a53U, 0x04550962U, 0xe0c0375fU, {16U, 16U, 16U, 16U}, {11U, 11U, 11U, 11U}, {0xfdffff4fU, 0xfff7fbffU, 0xee7ffdffU, 0xdfbbfdffU}, {0xfff7fbffU, 0xfdffff4fU, 0xdfbbfdffU, 0xee7ffdffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 28, 5, 3, 11, 5, 0xf8fefff7U, 0xdfffffffU, 0x7fffffffU, 0xfffdfbf7U, 0xa0000001U, 0xe4616943U, 0x0a9b362cU, 0xe8730649U, {5U, 5U, 5U, 5U}, {11U, 11U, 11U, 11U}, {0xf8fefff7U, 0xdfffffffU, 0x7fffffffU, 0xfffdfbf7U}, {0xdfffffffU, 0xf8fefff7U, 0xfffdfbf7U, 0x7fffffffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 4253, 12, 17, 5, 5, 3, 0xffffffffU, 0xbdffffffU, 0xe7fffffeU, 0xf3b7f7efU, 0xa0000001U, 0xe4d017d4U, 0xe32206c7U, 0xe65fe0eeU, {17U, 17U, 17U, 17U}, {5U, 5U, 5U, 5U}, {0xffffffffU, 0xbdffffffU, 0xe7fffffeU, 0xf3b7f7efU}, {0xbdffffffU, 0xffffffffU, 0xf3b7f7efU, 0xe7fffffeU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 13, 13, 5, 12, 3, 0x3ff7ff7fU, 0xffffdfdfU, 0xffbdbf7fU, 0xffffbeffU, 0x90000001U, 0x655f358eU, 0x996adc07U, 0x9ffd9669U, {13U, 13U, 13U, 13U}, {12U, 12U, 12U, 12U}, {0x3ff7ff7fU, 0xffffdfdfU, 0xffbdbf7fU, 0xffffbeffU}, {0xffffdfdfU, 0x3ff7ff7fU, 0xffffbeffU, 0xffbdbf7fU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 18, 13, 3, 17, 7, 0xbdf7ffffU, 0xfbdfbfffU, 0x7fdffb3fU, 0xffbe6befU, 0x20000001U, 0xb081cfbeU, 0xb3d5e047U, 0x100e4768U, {13U, 13U, 13U, 13U}, {17U, 17U, 17U, 17U}, {0xbdf7ffffU, 0xfbdfbfffU, 0x7fdffb3fU, 0xffbe6befU}, {0xfbdfbfffU, 0xbdf7ffffU, 0xffbe6befU, 0x7fdffb3fU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 4253, 11, 21, 3, 6, 1, 0xfffffea7U, 0xfffdff3aU, 0xbfffffcfU, 0xdff5ffedU, 0xa0000001U, 0x9405b85dU, 0xc60478f2U, 0xac65a61dU, {21U, 21U, 21U, 21U}, {6U, 6U, 6U, 6U}, {0xfffffea7U, 0xfffdff3aU, 0xbfffffcfU, 0xdff5ffedU}, {0xfffdff3aU, 0xfffffea7U, 0xdff5ffedU, 0xbfffffcfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 1, 12, 1, 7, 1, 0xffbfff7bU, 0xfabf7ffbU, 0xebfffefdU, 0x55fdfffeU, 0x00000001U, 0x324cb2feU, 0x442f545cU, 0x69f16b2bU, {12U, 12U, 12U, 12U}, {7U, 7U, 7U, 7U}, {0xffbfff7bU, 0xfabf7ffbU, 0xebfffefdU, 0x55fdfffeU}, {0xfabf7ffbU, 0xffbfff7bU, 0x55fdfffeU, 0xebfffefdU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 16, 18, 5, 7, 3, 0xdfbfffffU, 0xfb7e5fffU, 0xfffbdfffU, 0xdffbbfffU, 0x00000001U, 0x92040000U, 0x6265f31aU, 0x68abb978U, {18U, 18U, 18U, 18U}, {7U, 7U, 7U, 7U}, {0xdfbfffffU, 0xfb7e5fffU, 0xfffbdfffU, 0xdffbbfffU}, {0xfb7e5fffU, 0xdfbfffffU, 0xdffbbfffU, 0xfffbdfffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 13, 13, 5, 4, 3, 0xbf7febadU, 0xff3ff756U, 0xfafaf6f6U, 0xfffffffcU, 0x28000001U, 0xf2e5468cU, 0x7b320d81U, 0xd79d5d0dU, {13U, 13U, 13U, 13U}, {4U, 4U, 4U, 4U}, {0xbf7febadU, 0xff3ff756U, 0xfafaf6f6U, 0xfffffffcU}, {0xff3ff756U, 0xbf7febadU, 0xfffffffcU, 0xfafaf6f6U}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 4, 11, 1, 3, 3, 0xf7efdbfbU, 0xf7fffffbU, 0xfffefe7fU, 0xf7eb7fefU, 0x80000001U, 0x9026447eU, 0x1f279907U, 0x070b4438U, {11U, 11U, 11U, 11U}, {3U, 3U, 3U, 3U}, {0xf7efdbfbU, 0xf7fffffbU, 0xfffefe7fU, 0xf7eb7fefU}, {0xf7fffffbU, 0xf7efdbfbU, 0xf7eb7fefU, 0xfffefe7fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 32, 3, 3, 11, 3, 0xffffeffdU, 0xffffeff7U, 0xffffffefU, 0xffdfffffU, 0x00000001U, 0x3725b98fU, 0xd6e1adebU, 0xdfbc6de4U, {3U, 3U, 3U, 3U}, {11U, 11U, 11U, 11U}, {0xffffeffdU, 0xffffeff7U, 0xffffffefU, 0xffdfffffU}, {0xffffeff7U, 0xffffeffdU, 0xffdfffffU, 0xffffffefU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 9, 5, 3, 26, 1, 0xb9ffffdfU, 0xf9fdff7fU, 0xfffebffeU, 0xb6ff7ffdU, 0x10000001U, 0xbc129652U, 0xb4c6341bU, 0x13cb9597U, {5U, 5U, 5U, 5U}, {26U, 26U, 26U, 26U}, {0xb9ffffdfU, 0xf9fdff7fU, 0xfffebffeU, 0xb6ff7ffdU}, {0xf9fdff7fU, 0xb9ffffdfU, 0xb6ff7ffdU, 0xfffebffeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 32, 3, 3, 3, 5, 0xfffffeffU, 0xfffb9d7fU, 0xffdb9ff7U, 0xbff9fff7U, 0x60000001U, 0x0623be05U, 0x4730c7b7U, 0xe4dcb68eU, {3U, 3U, 3U, 3U}, {3U, 3U, 3U, 3U}, {0xfffffeffU, 0xfffb9d7fU, 0xffdb9ff7U, 0xbff9fff7U}, {0xfffb9d7fU, 0xfffffeffU, 0xbff9fff7U, 0xffdb9ff7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 4253, 32, 21, 3, 7, 1, 0xff3feefdU, 0xffffecffU, 0xfffffcfdU, 0xfffbadfdU, 0x70000001U, 0xb4dc9c74U, 0xfa71d735U, 0x2b9421bdU, {21U, 21U, 21U, 21U}, {7U, 7U, 7U, 7U}, {0xff3feefdU, 0xffffecffU, 0xfffffcfdU, 0xfffbadfdU}, {0xffffecffU, 0xff3feefdU, 0xfffbadfdU, 0xfffffcfdU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 13, 22, 1, 9, 3, 0xffffebfbU, 0x7fffbb7fU, 0xcbddfffbU, 0xfff7ff7fU, 0x00000001U, 0x179503c2U, 0x84579cd5U, 0x8bcf2f27U, {22U, 22U, 22U, 22U}, {9U, 9U, 9U, 9U}, {0xffffebfbU, 0x7fffbb7fU, 0xcbddfffbU, 0xfff7ff7fU}, {0x7fffbb7fU, 0xffffebfbU, 0xfff7ff7fU, 0xcbddfffbU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 27, 11, 5, 8, 3, 0xd73d23f7U, 0xef7fffffU, 0xeeffffdfU, 0xfffbf6cfU, 0x40000001U, 0xbec29ed7U, 0x722046bcU, 0x7d6ac005U, {11U, 11U, 11U, 11U}, {8U, 8U, 8U, 8U}, {0xd73d23f7U, 0xef7fffffU, 0xeeffffdfU, 0xfffbf6cfU}, {0xef7fffffU, 0xd73d23f7U, 0xfffbf6cfU, 0xeeffffdfU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 19, 26, 1, 19, 7, 0xefbfbff7U, 0xfb7fffffU, 0xfeeffdfbU, 0x7fbfebf9U, 0xb0000001U, 0xc49c2158U, 0xdd1db4e7U, 0x0da08972U, {26U, 26U, 26U, 26U}, {19U, 19U, 19U, 19U}, {0xefbfbff7U, 0xfb7fffffU, 0xfeeffdfbU, 0x7fbfebf9U}, {0xfb7fffffU, 0xefbfbff7U, 0x7fbfebf9U, 0xfeeffdfbU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 4253, 12, 29, 1, 1, 1, 0xfff7dffdU, 0xffdd7fbdU, 0xffffbff7U, 0xffff9dffU, 0xe0000001U, 0x8ecaba1bU, 0x4a61645bU, 0x6aac5023U, {29U, 29U, 29U, 29U}, {1U, 1U, 1U, 1U}, {0xfff7dffdU, 0xffdd7fbdU, 0xffffbff7U, 0xffff9dffU}, {0xffdd7fbdU, 0xfff7dffdU, 0xffff9dffU, 0xffffbff7U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 25, 22, 5, 19, 3, 0xefffffefU, 0xfbdfb3ffU, 0xfbf7f7fdU, 0xfef7fffbU, 0x80000001U, 0xacaa212fU, 0xaa5340c9U, 0x5ca836d2U, {22U, 22U, 22U, 22U}, {19U, 19U, 19U, 19U}, {0xefffffefU, 0xfbdfb3ffU, 0xfbf7f7fdU, 0xfef7fffbU}, {0xfbdfb3ffU, 0xefffffefU, 0xfef7fffbU, 0xfbf7f7fdU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 24, 2, 3, 7, 1, 0xef8ffffaU, 0xfffbfff8U, 0xf7ffbfe9U, 0x77ffbffdU, 0x20000001U, 0x8f3c8ae4U, 0xa3cf01c6U, 0xa91d27ddU, {2U, 2U, 2U, 2U}, {7U, 7U, 7U, 7U}, {0xef8ffffaU, 0xfffbfff8U, 0xf7ffbfe9U, 0x77ffbffdU}, {0xfffbfff8U, 0xef8ffffaU, 0x77ffbffdU, 0xf7ffbfe9U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 4253, 4, 14, 5, 9, 3, 0x7bfffff7U, 0xfff7fff3U, 0xff9fbf9eU, 0xfffdafffU, 0xc0000001U, 0x93fcf151U, 0x739a3cf7U, 0x72b647f3U, {14U, 14U, 14U, 14U}, {9U, 9U, 9U, 9U}, {0x7bfffff7U, 0xfff7fff3U, 0xff9fbf9eU, 0xfffdafffU}, {0xfff7fff3U, 0x7bfffff7U, 0xfffdafffU, 0xff9fbf9eU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 21, 20, 1, 19, 3, 0xffb7fffdU, 0x9fefedfdU, 0x9edb6ffbU, 0xbedeb7ffU, 0x40000001U, 0x9ed70b6bU, 0x661e1ca7U, 0xfd968985U, {20U, 20U, 20U, 20U}, {19U, 19U, 19U, 19U}, {0xffb7fffdU, 0x9fefedfdU, 0x9edb6ffbU, 0xbedeb7ffU}, {0x9fefedfdU, 0xffb7fffdU, 0xbedeb7ffU, 0x9edb6ffbU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 4253, 31, 15, 5, 14, 3, 0xefbce7ffU, 0xfeb7f77fU, 0xfff3ffbfU, 0xbfffb75fU, 0x48000001U, 0x836a277aU, 0xc4833eb6U, 0x22b0dc59U, {15U, 15U, 15U, 15U}, {14U, 14U, 14U, 14U}, {0xefbce7ffU, 0xfeb7f77fU, 0xfff3ffbfU, 0xbfffb75fU}, {0xfeb7f77fU, 0xefbce7ffU, 0xbfffb75fU, 0xfff3ffbfU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 

// 607.csv
{ 607, 2, 15, 3, 13, 3, 0xfdff37ffU, 0xef7f3f7dU, 0xff777b7dU, 0x7ff7fb2fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x5986f054U, {15U, 15U, 15U, 15U}, {13U, 13U, 13U, 13U}, {0xfdff37ffU, 0xef7f3f7dU, 0xff777b7dU, 0x7ff7fb2fU}, {0xef7f3f7dU, 0xfdff37ffU, 0x7ff7fb2fU, 0xff777b7dU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 2, 19, 1, 13, 1, 0xffbfffbfU, 0xcdffffffU, 0xeffbffedU, 0xfef8fbdcU, 0x00000001U, 0x00000000U, 0x80000000U, 0x30b388afU, {19U, 19U, 19U, 19U}, {13U, 13U, 13U, 13U}, {0xffbfffbfU, 0xcdffffffU, 0xeffbffedU, 0xfef8fbdcU}, {0xcdffffffU, 0xffbfffbfU, 0xfef8fbdcU, 0xeffbffedU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 13, 3, 11, 5, 0xdf7fffffU, 0xf7ff77ffU, 0xfffff2ffU, 0xffff473fU, 0x00000001U, 0x00000000U, 0x00000000U, 0xb5a64116U, {13U, 13U, 13U, 13U}, {11U, 11U, 11U, 11U}, {0xdf7fffffU, 0xf7ff77ffU, 0xfffff2ffU, 0xffff473fU}, {0xf7ff77ffU, 0xdf7fffffU, 0xffff473fU, 0xfffff2ffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 607, 3, 11, 3, 21, 1, 0x6fff7ffeU, 0xf5fffffeU, 0xbffffffeU, 0xafdfffffU, 0x00000001U, 0x00000000U, 0x10000000U, 0x6f6fec6aU, {11U, 11U, 11U, 11U}, {21U, 21U, 21U, 21U}, {0x6fff7ffeU, 0xf5fffffeU, 0xbffffffeU, 0xafdfffffU}, {0xf5fffffeU, 0x6fff7ffeU, 0xafdfffffU, 0xbffffffeU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 2, 15, 1, 3, 1, 0xff77fff7U, 0xfd77ffffU, 0xfffffdcfU, 0xfffffffaU, 0x00000001U, 0x00000000U, 0x20000000U, 0xd63bbe55U, {15U, 15U, 15U, 15U}, {3U, 3U, 3U, 3U}, {0xff77fff7U, 0xfd77ffffU, 0xfffffdcfU, 0xfffffffaU}, {0xfd77ffffU, 0xff77fff7U, 0xfffffffaU, 0xfffffdcfU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 7, 3, 2, 1, 0xbedf9cffU, 0xdfe6dfffU, 0xfdfffffaU, 0xdfff7efbU, 0x00000001U, 0x00000000U, 0x80000000U, 0xa52af51dU, {7U, 7U, 7U, 7U}, {2U, 2U, 2U, 2U}, {0xbedf9cffU, 0xdfe6dfffU, 0xfdfffffaU, 0xdfff7efbU}, {0xdfe6dfffU, 0xbedf9cffU, 0xdfff7efbU, 0xfdfffffaU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 14, 1, 1, 1, 0xffefffffU, 0x7e6ffa77U, 0xfff7fffaU, 0xfbbffffbU, 0x00000001U, 0x00000000U, 0x00000000U, 0x57810851U, {14U, 14U, 14U, 14U}, {1U, 1U, 1U, 1U}, {0xffefffffU, 0x7e6ffa77U, 0xfff7fffaU, 0xfbbffffbU}, {0x7e6ffa77U, 0xffefffffU, 0xfbbffffbU, 0xfff7fffaU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 2, 14, 1, 11, 1, 0x7bfffbffU, 0xfba7ffbfU, 0xf7fffe3fU, 0xf36fffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x170c7993U, {14U, 14U, 14U, 14U}, {11U, 11U, 11U, 11U}, {0x7bfffbffU, 0xfba7ffbfU, 0xf7fffe3fU, 0xf36fffffU}, {0xfba7ffbfU, 0x7bfffbffU, 0xf36fffffU, 0xf7fffe3fU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 14, 5, 3, 1, 0xffe7ffffU, 0xff6fefffU, 0x7f3fffffU, 0xff7feffbU, 0x00000001U, 0x00000000U, 0x40000000U, 0xf4112310U, {14U, 14U, 14U, 14U}, {3U, 3U, 3U, 3U}, {0xffe7ffffU, 0xff6fefffU, 0x7f3fffffU, 0xff7feffbU}, {0xff6fefffU, 0xffe7ffffU, 0xff7feffbU, 0x7f3fffffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 5, 3, 6, 3, 0xd3ddebfbU, 0xffdfffdfU, 0xffbf9bffU, 0xefefbfffU, 0x00000001U, 0x00000000U, 0x02000000U, 0x67e80003U, {5U, 5U, 5U, 5U}, {6U, 6U, 6U, 6U}, {0xd3ddebfbU, 0xffdfffdfU, 0xffbf9bffU, 0xefefbfffU}, {0xffdfffdfU, 0xd3ddebfbU, 0xefefbfffU, 0xffbf9bffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 3, 13, 1, 3, 3, 0x7fffd6beU, 0xffbefdfcU, 0xf7ff73f5U, 0x37fff7fbU, 0x00000001U, 0x00000000U, 0x80000000U, 0xcd8ababaU, {13U, 13U, 13U, 13U}, {3U, 3U, 3U, 3U}, {0x7fffd6beU, 0xffbefdfcU, 0xf7ff73f5U, 0x37fff7fbU}, {0xffbefdfcU, 0x7fffd6beU, 0x37fff7fbU, 0xf7ff73f5U}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 1, 12, 7, 9, 1, 0xfeffffdeU, 0xfe8fbfb7U, 0x63effffdU, 0x7fe7b3f5U, 0x00000001U, 0x00000000U, 0x00000000U, 0x694481ddU, {12U, 12U, 12U, 12U}, {9U, 9U, 9U, 9U}, {0xfeffffdeU, 0xfe8fbfb7U, 0x63effffdU, 0x7fe7b3f5U}, {0xfe8fbfb7U, 0xfeffffdeU, 0x7fe7b3f5U, 0x63effffdU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 13, 7, 5, 1, 0xfffff9b7U, 0xbfebffdbU, 0xe7dfeefeU, 0xf3ff3effU, 0x00000001U, 0x00000000U, 0x80000000U, 0x6e1c11beU, {13U, 13U, 13U, 13U}, {5U, 5U, 5U, 5U}, {0xfffff9b7U, 0xbfebffdbU, 0xe7dfeefeU, 0xf3ff3effU}, {0xbfebffdbU, 0xfffff9b7U, 0xf3ff3effU, 0xe7dfeefeU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 3, 6, 7, 1, 1, 0xfeffbff7U, 0xfffdd7ffU, 0x7ffffdffU, 0xfdffffffU, 0x00000001U, 0x00000000U, 0x00000000U, 0x047cd258U, {6U, 6U, 6U, 6U}, {1U, 1U, 1U, 1U}, {0xfeffbff7U, 0xfffdd7ffU, 0x7ffffdffU, 0xfdffffffU}, {0xfffdd7ffU, 0xfeffbff7U, 0xfdffffffU, 0x7ffffdffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 23, 1, 20, 1, 0xf5ffbfffU, 0xbffcfffeU, 0xffbffefeU, 0x7fffbeefU, 0x00000001U, 0x00000000U, 0x00000000U, 0xde196133U, {23U, 23U, 23U, 23U}, {20U, 20U, 20U, 20U}, {0xf5ffbfffU, 0xbffcfffeU, 0xffbffefeU, 0x7fffbeefU}, {0xbffcfffeU, 0xf5ffbfffU, 0x7fffbeefU, 0xffbffefeU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 2, 13, 1, 16, 3, 0xfffffbbfU, 0xfffdffefU, 0xefb97fffU, 0xdf7fffefU, 0x00000001U, 0x00000000U, 0x80000000U, 0x946dbc1cU, {13U, 13U, 13U, 13U}, {16U, 16U, 16U, 16U}, {0xfffffbbfU, 0xfffdffefU, 0xefb97fffU, 0xdf7fffefU}, {0xfffdffefU, 0xfffffbbfU, 0xdf7fffefU, 0xefb97fffU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 1, 17, 3, 22, 5, 0xdffffafeU, 0xdffffeffU, 0xfd77ffdfU, 0xfbfffffbU, 0x00000001U, 0x00000000U, 0x00000000U, 0xa7c70cc8U, {17U, 17U, 17U, 17U}, {22U, 22U, 22U, 22U}, {0xdffffafeU, 0xdffffeffU, 0xfd77ffdfU, 0xfbfffffbU}, {0xdffffeffU, 0xdffffafeU, 0xfbfffffbU, 0xfd77ffdfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 607, 1, 7, 3, 21, 3, 0xbffddfefU, 0xffffffd7U, 0x3ffffff7U, 0xfff5effbU, 0x00000001U, 0x00000000U, 0x40000000U, 0xe7aab0e7U, {7U, 7U, 7U, 7U}, {21U, 21U, 21U, 21U}, {0xbffddfefU, 0xffffffd7U, 0x3ffffff7U, 0xfff5effbU}, {0xffffffd7U, 0xbffddfefU, 0xfff5effbU, 0x3ffffff7U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 1, 5, 3, 15, 5, 0xffffdf7fU, 0xbfffed97U, 0xffeadcbfU, 0xf7ffbefbU, 0x00000001U, 0x00000000U, 0x00000000U, 0x7aa354e4U, {5U, 5U, 5U, 5U}, {15U, 15U, 15U, 15U}, {0xffffdf7fU, 0xbfffed97U, 0xffeadcbfU, 0xf7ffbefbU}, {0xbfffed97U, 0xffffdf7fU, 0xf7ffbefbU, 0xffeadcbfU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 607, 2, 10, 3, 23, 1, 0xffebccfdU, 0xaffffffdU, 0xf777bdffU, 0xaf7effffU, 0x00000001U, 0x00000000U, 0x80000000U, 0xc824d8c1U, {10U, 10U, 10U, 10U}, {23U, 23U, 23U, 23U}, {0xffebccfdU, 0xaffffffdU, 0xf777bdffU, 0xaf7effffU}, {0xaffffffdU, 0xffebccfdU, 0xaf7effffU, 0xf777bdffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 4, 3, 1, 1, 0xfe9fffefU, 0xffbffeffU, 0xbe57bdffU, 0xfefb3f5fU, 0x00000001U, 0x00000000U, 0x80000000U, 0x41129a62U, {4U, 4U, 4U, 4U}, {1U, 1U, 1U, 1U}, {0xfe9fffefU, 0xffbffeffU, 0xbe57bdffU, 0xfefb3f5fU}, {0xffbffeffU, 0xfe9fffefU, 0xfefb3f5fU, 0xbe57bdffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 2, 5, 3, 18, 5, 0xfcbeffcfU, 0x5bfebb7fU, 0xfff5efffU, 0xefb5eff9U, 0x00000001U, 0x00000000U, 0x00000000U, 0x4925021aU, {5U, 5U, 5U, 5U}, {18U, 18U, 18U, 18U}, {0xfcbeffcfU, 0x5bfebb7fU, 0xfff5efffU, 0xefb5eff9U}, {0x5bfebb7fU, 0xfcbeffcfU, 0xefb5eff9U, 0xfff5efffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 607, 1, 3, 3, 2, 3, 0xffffffffU, 0xffffeef7U, 0xbff6ff73U, 0xffe7f7d7U, 0x00000001U, 0x00000000U, 0x00000000U, 0x737556b0U, {3U, 3U, 3U, 3U}, {2U, 2U, 2U, 2U}, {0xffffffffU, 0xffffeef7U, 0xbff6ff73U, 0xffe7f7d7U}, {0xffffeef7U, 0xffffffffU, 0xffe7f7d7U, 0xbff6ff73U}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 3, 11, 5, 4, 1, 0x7fdfffffU, 0x7cddd7faU, 0xfebfdeeeU, 0xffbffef7U, 0x00000001U, 0x00000000U, 0x00000000U, 0xb4be184dU, {11U, 11U, 11U, 11U}, {4U, 4U, 4U, 4U}, {0x7fdfffffU, 0x7cddd7faU, 0xfebfdeeeU, 0xffbffef7U}, {0x7cddd7faU, 0x7fdfffffU, 0xffbffef7U, 0xfebfdeeeU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 2, 21, 5, 4, 1, 0xffbbf77fU, 0xff7ffbfbU, 0xbffff7f7U, 0xefecdfffU, 0x00000001U, 0x00000000U, 0x04000000U, 0xf361b11fU, {21U, 21U, 21U, 21U}, {4U, 4U, 4U, 4U}, {0xffbbf77fU, 0xff7ffbfbU, 0xbffff7f7U, 0xefecdfffU}, {0xff7ffbfbU, 0xffbbf77fU, 0xefecdfffU, 0xbffff7f7U}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 13, 3, 18, 7, 0xdfbf5fffU, 0xcffd7effU, 0xffbe3fffU, 0xfdbfff9fU, 0x00000001U, 0x00000000U, 0x80000000U, 0xd28269a7U, {13U, 13U, 13U, 13U}, {18U, 18U, 18U, 18U}, {0xdfbf5fffU, 0xcffd7effU, 0xffbe3fffU, 0xfdbfff9fU}, {0xcffd7effU, 0xdfbf5fffU, 0xfdbfff9fU, 0xffbe3fffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{9,10,11,4,13,14,15,8,31,31,31,12,31,31,31,31},{9,10,11,12,13,14,15,0,23,23,23,23,23,23,23,8}}, 
{ 607, 3, 11, 5, 12, 3, 0xebffefffU, 0xfbfffaffU, 0xf7fffeffU, 0x7ffefffeU, 0x00000001U, 0x00000000U, 0x40000000U, 0x39660c35U, {11U, 11U, 11U, 11U}, {12U, 12U, 12U, 12U}, {0xebffefffU, 0xfbfffaffU, 0xf7fffeffU, 0x7ffefffeU}, {0xfbfffaffU, 0xebffefffU, 0x7ffefffeU, 0xf7fffeffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 2, 11, 5, 6, 3, 0xffffffddU, 0xdff7bdf7U, 0xf3fff1ffU, 0xffff7ffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0x18ceeb85U, {11U, 11U, 11U, 11U}, {6U, 6U, 6U, 6U}, {0xffffffddU, 0xdff7bdf7U, 0xf3fff1ffU, 0xffff7ffeU}, {0xdff7bdf7U, 0xffffffddU, 0xffff7ffeU, 0xf3fff1ffU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{5,6,7,0,9,10,11,4,13,14,15,8,19,19,19,12},{13,14,15,0,1,2,3,4,19,19,19,8,9,10,11,12}}, 
{ 607, 2, 21, 3, 11, 5, 0xffff7ebfU, 0xfffd7fffU, 0xdffdfdffU, 0xffa57bdfU, 0x00000001U, 0x00000000U, 0x80000000U, 0x41d828efU, {21U, 21U, 21U, 21U}, {11U, 11U, 11U, 11U}, {0xffff7ebfU, 0xfffd7fffU, 0xdffdfdffU, 0xffa57bdfU}, {0xfffd7fffU, 0xffff7ebfU, 0xffa57bdfU, 0xdffdfdffU}, {3,21,21,21,7,0,1,2,11,4,5,6,15,8,9,10},{3,4,5,6,7,29,29,29,11,12,13,14,15,0,1,2},{11,4,5,6,15,8,9,10,29,12,13,14,29,29,29,29},{11,12,13,14,15,0,1,2,21,21,21,21,21,8,9,10}}, 
{ 607, 3, 20, 7, 7, 1, 0xbeffbffbU, 0xbffffffbU, 0xffffefffU, 0xeefbef3fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x2a5c3390U, {20U, 20U, 20U, 20U}, {7U, 7U, 7U, 7U}, {0xbeffbffbU, 0xbffffffbU, 0xffffefffU, 0xeefbef3fU}, {0xbffffffbU, 0xbeffbffbU, 0xeefbef3fU, 0xffffefffU}, {25,25,25,25,3,25,25,25,7,0,1,2,11,4,5,6},{7,25,25,25,25,25,25,25,15,0,1,2,3,4,5,6},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 18, 1, 7, 1, 0xf7ffb5fdU, 0xb76ff7fdU, 0xfdfff7dfU, 0xf9dffffeU, 0x00000001U, 0x00000000U, 0x00000000U, 0xd99b3c70U, {18U, 18U, 18U, 18U}, {7U, 7U, 7U, 7U}, {0xf7ffb5fdU, 0xb76ff7fdU, 0xfdfff7dfU, 0xf9dffffeU}, {0xb76ff7fdU, 0xf7ffb5fdU, 0xf9dffffeU, 0xfdfff7dfU}, {1,2,3,23,5,6,7,0,9,10,11,4,13,14,15,8},{1,2,3,4,5,6,7,31,9,10,11,12,13,14,15,0},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}, 
{ 607, 1, 21, 5, 19, 1, 0xbfff7effU, 0x7fffbfefU, 0xfffbff7fU, 0xeffeff6fU, 0x00000001U, 0x00000000U, 0x00000000U, 0x85b22d17U, {21U, 21U, 21U, 21U}, {19U, 19U, 19U, 19U}, {0xbfff7effU, 0x7fffbfefU, 0xfffbff7fU, 0xeffeff6fU}, {0x7fffbfefU, 0xbfff7effU, 0xeffeff6fU, 0xfffbff7fU}, {27,27,27,27,1,2,3,27,5,6,7,0,9,10,11,4},{5,6,7,27,27,27,27,27,13,14,15,0,1,2,3,4},{7,0,1,2,11,4,5,6,15,8,9,10,17,12,13,14},{15,0,1,2,3,4,5,6,17,8,9,10,11,12,13,14}}


};


#endif // PPU




#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* _MC_RAND_SFMT_H */
